diff --git a/LICENSE b/LICENSE
index 814fc7b..52998af 100644
--- a/LICENSE
+++ b/LICENSE
@@ -1,4 +1,4 @@
-This software is Copyright (C) 2008 Mike McCauley. Use is subject to license
+This software is Copyright (C) Mike McCauley. Use is subject to license
conditions. The main licensing options available are GPL V3 or Commercial:
Open Source Licensing GPL V3
@@ -14,4 +14,4 @@ Commercial Licensing
This is the appropriate option if you are creating proprietary applications
and you are not prepared to distribute and share the source code of your
-application. Contact info@open.com.au for details.
+application. Contact info@airspayce for details.
diff --git a/extras/doc/annotated.html b/extras/doc/annotated.html
index 7f48b6f..e60691d 100644
--- a/extras/doc/annotated.html
+++ b/extras/doc/annotated.html
@@ -1,9 +1,9 @@
-
+
-
+
AccelStepper: Class List
@@ -26,14 +26,15 @@
-
+
+/* @license-end */
@@ -50,9 +51,7 @@ $(function() {
-Generated by
-
- 1.8.13
+Generated by 1.9.1
diff --git a/extras/doc/classAccelStepper-members.html b/extras/doc/classAccelStepper-members.html
index 44b31e5..49f5140 100644
--- a/extras/doc/classAccelStepper-members.html
+++ b/extras/doc/classAccelStepper-members.html
@@ -1,9 +1,9 @@
-
+
-
+
AccelStepper: Member List
@@ -26,14 +26,15 @@
-
+
+/* @license-end */
@@ -45,9 +46,11 @@ $(function() {
This is the complete list of members for AccelStepper, including all inherited members.
Symbolic names for number of pins. Use this in the pins argument the AccelStepper constructor to provide a symbolic name for the number of pins to use. More...
-
+ }
+
Symbolic names for number of pins. Use this in the pins argument the AccelStepper constructor to provide a symbolic name for the number of pins to use. More...
This defines a single 2 or 4 pin stepper motor, or stepper moter with fdriver chip, with optional acceleration, deceleration, absolute positioning commands etc. Multiple simultaneous steppers are supported, all moving at different speeds and accelerations.
Operation
This module operates by computing a step time in microseconds. The step time is recomputed after each step and after speed and acceleration parameters are changed by the caller. The time of each step is recorded in microseconds. The run() function steps the motor once if a new step is due. The run() function must be called frequently until the motor is in the desired position, after which time run() will do nothing.
Positioning
Positions are specified by a signed long integer. At construction time, the current position of the motor is consider to be 0. Positive positions are clockwise from the initial position; negative positions are anticlockwise. The current position can be altered for instance after initialization positioning.
-
Caveats
This is an open loop controller: If the motor stalls or is oversped, AccelStepper will not have a correct idea of where the motor really is (since there is no feedback of the motor's real position. We only know where we think it is, relative to the initial starting point).
+
Caveats
This is an open loop controller: If the motor stalls or is oversped, AccelStepper will not have a correct idea of where the motor really is (since there is no feedback of the motor's real position. We only know where we think it is, relative to the initial starting point).
Performance
The fastest motor speed that can be reliably supported is about 4000 steps per second at a clock frequency of 16 MHz on Arduino such as Uno etc. Faster processors can support faster stepping speeds. However, any speed less than that down to very slow speeds (much less than one per second) are also supported, provided the run() function is called frequently enough to step the motor whenever required for the speed set. Calling setAcceleration() is expensive, since it requires a square root to be calculated.
Gregor Christandl reports that with an Arduino Due and a simple test program, he measured 43163 steps per second using runSpeed(), and 16214 steps per second using run();
Member Enumeration Documentation
@@ -212,7 +225,7 @@ Protected Attributes
-
Symbolic names for number of pins. Use this in the pins argument the AccelStepper constructor to provide a symbolic name for the number of pins to use.
+
Symbolic names for number of pins. Use this in the pins argument the AccelStepper constructor to provide a symbolic name for the number of pins to use.
Enumerator
FUNCTION
Use the functional interface, implementing your own driver functions (internal use only)
@@ -284,9 +297,9 @@ Protected Attributes
Constructor. You can have multiple simultaneous steppers, all moving at different speeds and accelerations, provided you call their run() functions at frequent enough intervals. Current Position is set to 0, target position is set to 0. MaxSpeed and Acceleration default to 1.0. The motor pins will be initialised to OUTPUT mode during the constructor by a call to enableOutputs().
Parameters
-
[in]
interface
Number of pins to interface to. Integer values are supported, but it is preferred to use the MotorInterfaceType symbolic names. AccelStepper::DRIVER (1) means a stepper driver (with Step and Direction pins). If an enable line is also needed, call setEnablePin() after construction. You may also invert the pins using setPinsInverted(). Caution: DRIVER implements a blocking delay of minPulseWidth microseconds (default 1us) for each step. You can change this with setMinPulseWidth(). AccelStepper::FULL2WIRE (2) means a 2 wire stepper (2 pins required). AccelStepper::FULL3WIRE (3) means a 3 wire stepper, such as HDD spindle (3 pins required). AccelStepper::FULL4WIRE (4) means a 4 wire stepper (4 pins required). AccelStepper::HALF3WIRE (6) means a 3 wire half stepper, such as HDD spindle (3 pins required) AccelStepper::HALF4WIRE (8) means a 4 wire half stepper (4 pins required) Defaults to AccelStepper::FULL4WIRE (4) pins.
-
[in]
pin1
Arduino digital pin number for motor pin 1. Defaults to pin 2. For a AccelStepper::DRIVER (interface==1), this is the Step input to the driver. Low to high transition means to step)
-
[in]
pin2
Arduino digital pin number for motor pin 2. Defaults to pin 3. For a AccelStepper::DRIVER (interface==1), this is the Direction input the driver. High means forward.
+
[in]
interface
Number of pins to interface to. Integer values are supported, but it is preferred to use the MotorInterfaceType symbolic names. AccelStepper::DRIVER (1) means a stepper driver (with Step and Direction pins). If an enable line is also needed, call setEnablePin() after construction. You may also invert the pins using setPinsInverted(). Caution: DRIVER implements a blocking delay of minPulseWidth microseconds (default 1us) for each step. You can change this with setMinPulseWidth(). AccelStepper::FULL2WIRE (2) means a 2 wire stepper (2 pins required). AccelStepper::FULL3WIRE (3) means a 3 wire stepper, such as HDD spindle (3 pins required). AccelStepper::FULL4WIRE (4) means a 4 wire stepper (4 pins required). AccelStepper::HALF3WIRE (6) means a 3 wire half stepper, such as HDD spindle (3 pins required) AccelStepper::HALF4WIRE (8) means a 4 wire half stepper (4 pins required) Defaults to AccelStepper::FULL4WIRE (4) pins.
+
[in]
pin1
Arduino digital pin number for motor pin 1. Defaults to pin 2. For a AccelStepper::DRIVER (interface==1), this is the Step input to the driver. Low to high transition means to step)
+
[in]
pin2
Arduino digital pin number for motor pin 2. Defaults to pin 3. For a AccelStepper::DRIVER (interface==1), this is the Direction input the driver. High means forward.
[in]
pin3
Arduino digital pin number for motor pin 3. Defaults to pin 4.
[in]
pin4
Arduino digital pin number for motor pin 4. Defaults to pin 5.
[in]
enable
If this is true (the default), enableOutputs() will be called to enable the output pins at construction time.
Poll the motor and step it if a step is due, implementing accelerations and decelerations to achieve the target position. You must call this as frequently as possible, but at least once per minimum step time interval, preferably in your main loop. Note that each call to run() will make at most one step, and then only when a step is due, based on the current speed and the time since the last step.
Returns
true if the motor is still running to the target position.
Poll the motor and step it if a step is due, implementing a constant speed as set by the most recent call to setSpeed(). You must call this as frequently as possible, but at least once per step interval,
Runs at the currently selected speed until the target position is reached. Does not implement accelerations.
Returns
true if it stepped
+
Executes runSpeed() unless the targetPosition is reached. This function needs to be called often just like runSpeed() or run(). Will step the motor if a step is required at the currently selected speed unless the target position has been reached. Does not implement accelerations.
Sets the minimum pulse width allowed by the stepper driver. The minimum practical pulse width is approximately 20 microseconds. Times less than 20 microseconds will usually result in 20 microseconds or so.
Called to execute a step on a 4 pin half-steper motor. Only called when a new step is required. Subclasses may override to implement new stepping interfaces. The default sets or clears the outputs of pin1, pin2, pin3, pin4.
Parameters
+
Called to execute a step on a 4 pin half-stepper motor. Only called when a new step is required. Subclasses may override to implement new stepping interfaces. The default sets or clears the outputs of pin1, pin2, pin3, pin4.
This is the Arduino AccelStepper library. It provides an object-oriented interface for 2, 3 or 4 pin stepper motors and motor drivers.
+
This is the Arduino AccelStepper library. It provides an object-oriented interface for 2, 3 or 4 pin stepper motors and motor drivers.
The standard Arduino IDE includes the Stepper library (http://arduino.cc/en/Reference/Stepper) for stepper motors. It is perfectly adequate for simple, single motor applications.
-
AccelStepper significantly improves on the standard Arduino Stepper library in several ways:
+
AccelStepper significantly improves on the standard Arduino Stepper library in several ways:
Supports acceleration and deceleration
Supports multiple simultaneous steppers, with independent concurrent stepping on each stepper
Most API functions never delay() or block (unless otherwise stated)
Example Arduino programs are included to show the main modes of use.
You can also find online help and discussion at http://groups.google.com/group/accelstepper Please use that group for all questions and discussions on this topic. Do not contact the author directly, unless it is to discuss commercial licensing. Before asking a question or reporting a bug, please read
Tested on Arduino Diecimila and Mega with arduino-0018 & arduino-0021 on OpenSuSE 11.1 and avr-libc-1.6.1-1.15, cross-avr-binutils-2.19-9.1, cross-avr-gcc-4.1.3_20080612-26.5. Tested on Teensy http://www.pjrc.com/teensy including Teensy 3.1 built using Arduino IDE 1.0.5 with teensyduino addon 1.18 and later.
Installation
Install in the usual way: unzip the distribution zip file to the libraries sub-folder of your sketchbook.
This library is offered under a free GPL license for those who want to use it that way. We try hard to keep it up to date, fix bugs and to provide free support. If this library has helped you save time or money, please consider donating at http://www.airspayce.com or here:
Trademarks
-
AccelStepper is a trademark of AirSpayce Pty Ltd. The AccelStepper mark was first used on April 26 2010 for international trade, and is used only in relation to motor control hardware and software. It is not to be confused with any other similar marks covering other goods and services.
+
AccelStepper is a trademark of AirSpayce Pty Ltd. The AccelStepper mark was first used on April 26 2010 for international trade, and is used only in relation to motor control hardware and software. It is not to be confused with any other similar marks covering other goods and services.
Copyright
-
This software is Copyright (C) 2010-2018 Mike McCauley. Use is subject to license conditions. The main licensing options available are GPL V3 or Commercial:
+
This software is Copyright (C) 2010-2021 Mike McCauley. Use is subject to license conditions. The main licensing options available are GPL V3 or Commercial:
Open Source Licensing GPL V3
This is the appropriate option if you want to share the source code of your application with everyone you distribute it to, and you also want to give them the right to share who uses it. If you wish to use this software under Open Source Licensing, you must contribute all your source code to the open source community in accordance with the GPL Version 23 when your application is distributed. See https://www.gnu.org/licenses/gpl-3.0.html
Commercial Licensing
This is the appropriate option if you are creating proprietary applications and you are not prepared to distribute and share the source code of your application. To purchase a commercial license, contact info@.nosp@m.airs.nosp@m.payce.nosp@m..com
Revision History
@@ -118,7 +123,7 @@ $(function() {
1.19 Added MotorInterfaceType and symbolic names for the number of pins to use for the motor interface. Updated examples to suit. Replaced individual pin assignment variables _pin1, _pin2 etc with array _pin[4]. _pins member changed to _interface. Added _pinInverted array to simplify pin inversion operations. Added new function setOutputPins() which sets the motor output pins. It can be overridden in order to provide, say, serial output instead of parallel output Some refactoring and code size reduction.
-1.20 Improved documentation and examples to show need for correctly specifying AccelStepper::FULL4WIRE and friends.
+1.20 Improved documentation and examples to show need for correctly specifying AccelStepper::FULL4WIRE and friends.
1.21 Fixed a problem where desiredSpeed could compute the wrong step acceleration when _speed was small but non-zero. Reported by Brian Schmalz. Precompute sqrt_twoa to improve performance and max possible stepping speed
@@ -162,40 +167,40 @@ $(function() {
1.41 Fixed an error in the acceleration calculations, resulting in acceleration of haldf the intended value
-1.42 Improved support for FULL3WIRE and HALF3WIRE output pins. These changes were in Yuri's original contribution but did not make it into production.
+1.42 Improved support for FULL3WIRE and HALF3WIRE output pins. These changes were in Yuri's original contribution but did not make it into production.
-1.43 Added DualMotorShield example. Shows how to use AccelStepper to control 2 x 2 phase steppers using the Itead Studio Arduino Dual Stepper Motor Driver Shield model IM120417015.
+1.43 Added DualMotorShield example. Shows how to use AccelStepper to control 2 x 2 phase steppers using the Itead Studio Arduino Dual Stepper Motor Driver Shield model IM120417015.
-1.44 examples/DualMotorShield/DualMotorShield.ino examples/DualMotorShield/DualMotorShield.pde was missing from the distribution.
+1.44 examples/DualMotorShield/DualMotorShield.ino examples/DualMotorShield/DualMotorShield.pde was missing from the distribution.
-1.45 Fixed a problem where if setAcceleration was not called, there was no default acceleration. Reported by Michael Newman.
+1.45 Fixed a problem where if setAcceleration was not called, there was no default acceleration. Reported by Michael Newman.
-1.45 Fixed inaccuracy in acceleration rate by using Equation 15, suggested by Sebastian Gracki.
- Performance improvements in runSpeed suggested by Jaakko Fagerlund.
+1.45 Fixed inaccuracy in acceleration rate by using Equation 15, suggested by Sebastian Gracki.
+ Performance improvements in runSpeed suggested by Jaakko Fagerlund.
-1.46 Fixed error in documentation for runToPosition(). Reinstated time calculations in runSpeed() since new version is reported not to work correctly under some circumstances. Reported by Oleg V Gavva.
+1.46 Fixed error in documentation for runToPosition(). Reinstated time calculations in runSpeed() since new version is reported not to work correctly under some circumstances. Reported by Oleg V Gavva.
-1.48 2015-08-25 Added new class MultiStepper that can manage multiple AccelSteppers, and cause them all to move to selected positions at such a (constant) speed that they all arrive at their target position at the same time. Suitable for X-Y flatbeds etc.
- Added new method maxSpeed() to AccelStepper to return the currently configured maxSpeed.
+1.48 2015-08-25 Added new class MultiStepper that can manage multiple AccelSteppers, and cause them all to move to selected positions at such a (constant) speed that they all arrive at their target position at the same time. Suitable for X-Y flatbeds etc.
+ Added new method maxSpeed() to AccelStepper to return the currently configured maxSpeed.
-1.49 2016-01-02 Testing with VID28 series instrument stepper motors and EasyDriver. OK, although with light pointers and slow speeds like 180 full steps per second the motor movement can be erratic, probably due to some mechanical resonance. Best to accelerate through this speed.
- Added isRunning().
+1.49 2016-01-02 Testing with VID28 series instrument stepper motors and EasyDriver. OK, although with light pointers and slow speeds like 180 full steps per second the motor movement can be erratic, probably due to some mechanical resonance. Best to accelerate through this speed.
+ Added isRunning().
-1.50 2016-02-25 AccelStepper::disableOutputs now sets the enable pion to OUTPUT mode if the enable pin is defined. Patch from Piet De Jong.
- Added notes about the fact that AFMotor_* examples do not work with Adafruit Motor Shield V2.
+1.50 2016-02-25 AccelStepper::disableOutputs now sets the enable pion to OUTPUT mode if the enable pin is defined. Patch from Piet De Jong.
+ Added notes about the fact that AFMotor_* examples do not work with Adafruit Motor Shield V2.
1.51 2016-03-24 Fixed a problem reported by gregor: when resetting the stepper motor position using setCurrentPosition() the stepper speed is reset by setting _stepInterval to 0, but _speed is not reset. this results in the stepper motor not starting again when calling setSpeed() with the same speed the stepper was set to before.
-1.52 2016-08-09 Added MultiStepper to keywords.txt. Improvements to efficiency of AccelStepper::runSpeed() as suggested by David Grayson. Improvements to speed accuracy as suggested by David Grayson.
+1.52 2016-08-09 Added MultiStepper to keywords.txt. Improvements to efficiency of AccelStepper::runSpeed() as suggested by David Grayson. Improvements to speed accuracy as suggested by David Grayson.
1.53 2016-08-14 Backed out Improvements to speed accuracy from 1.52 as it did not work correctly.
@@ -213,14 +218,22 @@ $(function() {
1.60 2020-03-07 Release under GPL V3
-1.61 2020-04-20 Added yield() call in runToPosition(), so that platforms like esp8266 dont hang/crash during long runs.
+1.61 2020-04-20 Added yield() call in runToPosition(), so that platforms like esp8266 dont hang/crash during long runs.
+
+1.62 2022-05-22 Added link to AccelStepper - The Missing Manual.
+ Fixed a problem when setting the maxSpeed to 1.0 due to incomplete initialisation. Reported by Olivier Pécheux.
+
+
+1.63 2022-06-30 Added virtual destructor at the request of Jan.
+
+
+1.64 2022-10-31 Patch courtesy acwest: Changes to make AccelStepper more subclassable. These changes are largely oriented to implementing new step-scheduling algorithms.
+
-Generated by
-
- 1.8.13
+Generated by 1.9.1
diff --git a/keywords.txt b/keywords.txt
index e77e0f1..85d801c 100644
--- a/keywords.txt
+++ b/keywords.txt
@@ -35,6 +35,7 @@ setMinPulseWidth KEYWORD2
setEnablePin KEYWORD2
setPinsInverted KEYWORD2
maxSpeed KEYWORD2
+isRunning KEYWORD2
#######################################
# Constants (LITERAL1)
#######################################
diff --git a/library.properties b/library.properties
index 359401d..62debd6 100755
--- a/library.properties
+++ b/library.properties
@@ -1,5 +1,5 @@
name=AccelStepper
-version=1.61
+version=1.64
author=Mike McCauley
maintainer=Patrick Wasp
sentence=Allows Arduino boards to control a variety of stepper motors.
diff --git a/src/AccelStepper.cpp b/src/AccelStepper.cpp
index 093d56d..54535cc 100644
--- a/src/AccelStepper.cpp
+++ b/src/AccelStepper.cpp
@@ -1,6 +1,6 @@
// AccelStepper.cpp
//
-// Copyright (C) 2009-2013 Mike McCauley
+// Copyright (C) 2009-2020 Mike McCauley
// $Id: AccelStepper.cpp,v 1.24 2020/04/20 00:15:03 mikem Exp mikem $
#include "AccelStepper.h"
@@ -94,7 +94,8 @@ void AccelStepper::setCurrentPosition(long position)
_speed = 0.0;
}
-void AccelStepper::computeNewSpeed()
+// Subclasses can override
+unsigned long AccelStepper::computeNewSpeed()
{
long distanceTo = distanceToGo(); // +ve is clockwise from curent location
@@ -106,7 +107,7 @@ void AccelStepper::computeNewSpeed()
_stepInterval = 0;
_speed = 0.0;
_n = 0;
- return;
+ return _stepInterval;
}
if (distanceTo > 0)
@@ -174,6 +175,7 @@ void AccelStepper::computeNewSpeed()
Serial.println(stepsToStop);
Serial.println("-----");
#endif
+ return _stepInterval;
}
// Run the motor to implement speed and acceleration in order to proceed to the target position
@@ -193,7 +195,7 @@ AccelStepper::AccelStepper(uint8_t interface, uint8_t pin1, uint8_t pin2, uint8_
_currentPos = 0;
_targetPos = 0;
_speed = 0.0;
- _maxSpeed = 1.0;
+ _maxSpeed = 0.0;
_acceleration = 0.0;
_sqrt_twoa = 1.0;
_stepInterval = 0;
@@ -220,6 +222,7 @@ AccelStepper::AccelStepper(uint8_t interface, uint8_t pin1, uint8_t pin2, uint8_
enableOutputs();
// Some reasonable default
setAcceleration(1);
+ setMaxSpeed(1);
}
AccelStepper::AccelStepper(void (*forward)(), void (*backward)())
@@ -228,7 +231,7 @@ AccelStepper::AccelStepper(void (*forward)(), void (*backward)())
_currentPos = 0;
_targetPos = 0;
_speed = 0.0;
- _maxSpeed = 1.0;
+ _maxSpeed = 0.0;
_acceleration = 0.0;
_sqrt_twoa = 1.0;
_stepInterval = 0;
@@ -254,6 +257,7 @@ AccelStepper::AccelStepper(void (*forward)(), void (*backward)())
_pinInverted[i] = 0;
// Some reasonable default
setAcceleration(1);
+ setMaxSpeed(1);
}
void AccelStepper::setMaxSpeed(float speed)
@@ -295,6 +299,11 @@ void AccelStepper::setAcceleration(float acceleration)
}
}
+float AccelStepper::acceleration()
+{
+ return _acceleration;
+}
+
void AccelStepper::setSpeed(float speed)
{
if (speed == _speed)
@@ -350,6 +359,24 @@ void AccelStepper::step(long step)
}
}
+long AccelStepper::stepForward()
+{
+ // Clockwise
+ _currentPos += 1;
+ step(_currentPos);
+ _lastStepTime = micros();
+ return _currentPos;
+}
+
+long AccelStepper::stepBackward()
+{
+ // Counter-clockwise
+ _currentPos -= 1;
+ step(_currentPos);
+ _lastStepTime = micros();
+ return _currentPos;
+}
+
// You might want to override this to implement eg serial output
// bit 0 of the mask corresponds to _pin[0]
// bit 1 of the mask corresponds to _pin[1]
diff --git a/src/AccelStepper.h b/src/AccelStepper.h
index 5ab5aba..42ee37d 100644
--- a/src/AccelStepper.h
+++ b/src/AccelStepper.h
@@ -23,7 +23,7 @@
/// The latest version of this documentation can be downloaded from
/// http://www.airspayce.com/mikem/arduino/AccelStepper
/// The version of the package that this documentation refers to can be downloaded
-/// from http://www.airspayce.com/mikem/arduino/AccelStepper/AccelStepper-1.61.zip
+/// from http://www.airspayce.com/mikem/arduino/AccelStepper/AccelStepper-1.64.zip
///
/// Example Arduino programs are included to show the main modes of use.
///
@@ -35,6 +35,10 @@
/// - http://www.catb.org/esr/faqs/smart-questions.html
/// - http://www.chiark.greenend.org.uk/~shgtatham/bugs.html
///
+/// Beginners to C++ and stepper motors in general may find this helpful:
+/// - https://hackaday.io/project/183279-accelstepper-the-missing-manual
+/// - https://hackaday.io/project/183713-using-the-arduino-accelstepper-library
+///
/// Tested on Arduino Diecimila and Mega with arduino-0018 & arduino-0021
/// on OpenSuSE 11.1 and avr-libc-1.6.1-1.15,
/// cross-avr-binutils-2.19-9.1, cross-avr-gcc-4.1.3_20080612-26.5.
@@ -81,7 +85,7 @@
///
/// \par Copyright
///
-/// This software is Copyright (C) 2010-2018 Mike McCauley. Use is subject to license
+/// This software is Copyright (C) 2010-2021 Mike McCauley. Use is subject to license
/// conditions. The main licensing options available are GPL V3 or Commercial:
///
/// \par Open Source Licensing GPL V3
@@ -249,9 +253,18 @@
/// \version 1.61 2020-04-20
/// Added yield() call in runToPosition(), so that platforms like esp8266 dont hang/crash
/// during long runs.
+/// \version 1.62 2022-05-22
+/// Added link to AccelStepper - The Missing Manual.
+/// Fixed a problem when setting the maxSpeed to 1.0 due to incomplete initialisation.
+/// Reported by Olivier Pécheux.
+/// \version 1.63 2022-06-30
+/// Added virtual destructor at the request of Jan.
+/// \version 1.64 2022-10-31
+/// Patch courtesy acwest: Changes to make AccelStepper more subclassable. These changes are
+/// largely oriented to implementing new step-scheduling algorithms.
///
-/// \author Mike McCauley (mikem@airspayce.com) DO NOT CONTACT THE AUTHOR DIRECTLY: USE THE LISTS
-// Copyright (C) 2009-2013 Mike McCauley
+/// \author Mike McCauley (mikem@airspayce.com) DO NOT CONTACT THE AUTHOR DIRECTLY: USE THE GOOGLE GROUP
+// Copyright (C) 2009-2020 Mike McCauley
// $Id: AccelStepper.h,v 1.28 2020/04/20 00:15:03 mikem Exp mikem $
#ifndef AccelStepper_h
@@ -429,6 +442,11 @@ public:
/// root to be calculated. Dont call more ofthen than needed
void setAcceleration(float acceleration);
+ /// Returns the acceleration/deceleration rate configured for this stepper
+ /// that was previously set by setAcceleration();
+ /// \return The currently configured acceleration/deceleration
+ float acceleration();
+
/// Sets the desired constant speed for use with runSpeed().
/// \param[in] speed The desired constant speed in steps per
/// second. Positive is clockwise. Speeds of more than 1000 steps per
@@ -471,7 +489,10 @@ public:
/// position. Dont use this in event loops, since it blocks.
void runToPosition();
- /// Runs at the currently selected speed until the target position is reached.
+ /// Executes runSpeed() unless the targetPosition is reached.
+ /// This function needs to be called often just like runSpeed() or run().
+ /// Will step the motor if a step is required at the currently selected
+ /// speed unless the target position has been reached.
/// Does not implement accelerations.
/// \return true if it stepped
boolean runSpeedToPosition();
@@ -533,6 +554,8 @@ public:
/// \return true if the speed is not zero or not at the target position
bool isRunning();
+ /// Virtual destructor to prevent warnings during delete
+ virtual ~AccelStepper() {};
protected:
/// \brief Direction indicator
@@ -551,7 +574,8 @@ protected:
/// \li after change to acceleration through setAcceleration()
/// \li after change to target position (relative or absolute) through
/// move() or moveTo()
- void computeNewSpeed();
+ /// \return the new step interval
+ virtual unsigned long computeNewSpeed();
/// Low level function to set the motor output pins
/// bit 0 of the mask corresponds to _pin[0]
@@ -566,6 +590,16 @@ protected:
/// number of pins defined for the stepper.
/// \param[in] step The current step phase number (0 to 7)
virtual void step(long step);
+
+ /// Called to execute a clockwise(+) step. Only called when a new step is
+ /// required. This increments the _currentPos and calls step()
+ /// \return the updated current position
+ long stepForward();
+
+ /// Called to execute a counter-clockwise(-) step. Only called when a new step is
+ /// required. This decrements the _currentPos and calls step()
+ /// \return the updated current position
+ long stepBackward();
/// Called to execute a step using stepper functions (pins = 0) Only called when a new step is
/// required. Calls _forward() or _backward() to perform the step
@@ -607,7 +641,7 @@ protected:
/// \param[in] step The current step phase number (0 to 7)
virtual void step6(long step);
/// Called to execute a step on a 4 pin half-steper motor. Only called when a new step is
/// required. Subclasses may override to implement new stepping
/// interfaces. The default sets or clears the outputs of pin1, pin2,
/// pin3, pin4.
@@ -618,6 +652,10 @@ protected:
/// Protected because some peoples subclasses need it to be so
boolean _direction; // 1 == CW
+ /// The current interval between steps in microseconds.
+ /// 0 means the motor is currently stopped with _speed == 0
+ unsigned long _stepInterval;
+
private:
/// Number of pins on the stepper motor. Permits 2 or 4. 2 pins is a
/// bipolar, and 4 pins is a unipolar.
@@ -650,10 +688,6 @@ private:
float _acceleration;
float _sqrt_twoa; // Precomputed sqrt(2*_acceleration)
- /// The current interval between steps in microseconds.
- /// 0 means the motor is currently stopped with _speed == 0
- unsigned long _stepInterval;
-
/// The last step time in microseconds
unsigned long _lastStepTime;
@@ -607,7 +641,7 @@ protected:
/// \param[in] step The current step phase number (0 to 7)
virtual void step6(long step);
- /// Called to execute a step on a 4 pin half-steper motor. Only called when a new step is
+ /// Called to execute a step on a 4 pin half-stepper motor. Only called when a new step is
/// required. Subclasses may override to implement new stepping
/// interfaces. The default sets or clears the outputs of pin1, pin2,